Attribute VB_Name = "AutoCapBas"
'                                                  Automatic Capitalization
'                                                         Charles E. Isom
'                                                       CEIsom@aol.com
'                                                          (254)519-4363
'   I wrote this procedure because I got tired of remembering to press the <Shift> key to
'   capitalize words when inputting data for my database programs.  Maybe you can use
'   it also.

Option Explicit

Public strStringToCap As String           'Variable to hold the string to capitalize
Public blnAutoCap As Boolean            'Flag to capitalize automatically
'*****************************************************************************************************************
'   Purpose:  Capitalizes a string if the AutoCap flag is set to True.
'   Can also use to change any single quotes in the input to a graves mark to
'   get around any potential problems caused by the Jet Database Engine searching for strings
'   containing single or double quotes.  Your own setup is left up to you.
'   Inputs:     strStringToCap:    The string, data, etc to capitalize.  This string is checked character
'                                               by character and the current character is capitalized or changed
'                                               based on the criteria you give.  The criteria used is for demo
'                                               purposes only.
'   Returns:  The capitalized string.
'******************************************************************************************************************
Public Sub AutoCapData(strStringToCap As String)
    On Error GoTo ErrHandler
    Dim intLetterToCap As Integer                   'Location of current letter
    Dim intSep As Integer                                'Location of period bwtn 2 letters, such as: u.i
    Dim intTextLen As Integer                         'Length of the string
    Dim strLastChar As String * 1                     'The very last character in the string
    Dim BeforeSep As String                           'All characters from the start to the intSep period
    Dim AfterSep As String                              'All characters from the intSep period to the end
    
    '   Check the last character in the string.  If it is a single quote, change it to a graves mark.
    '   This is done if the AutoCap flag is on or off.  Without this, the last character is ignored.
    strLastChar = Right$(strStringToCap, 1)
    If strLastChar = "'" Then
        strLastChar = "`"
        strStringToCap = Left$(strStringToCap, Len(strStringToCap) - 1)
        strStringToCap = strStringToCap & strLastChar
    End If
    
    
    Select Case blnAutoCap
        Case False
            '   If the AutoCap flag is not set then
            '   change the all single quotes to a graves marks
            '   and exit the procedure
            For intLetterToCap = 1 To Len(strStringToCap) - 1
                If Mid$(strStringToCap, intLetterToCap, 1) = "'" Then
                    Mid$(strStringToCap, intLetterToCap, 1) = "`"
                End If
            Next
            Exit Sub
        Case True
            '   If the AutoCap flag is set then specify your changes, letter by letter
            If strStringToCap <> "" Then
                Mid$(strStringToCap, 1, 1) = UCase$(Mid$(strStringToCap, 1, 1))
                For intLetterToCap = 1 To Len(strStringToCap) - 1
                
                '   If a single quote is located between 2 letters and the word is not possessive then
                '   capitalize the letters before and after the single quote
                '   or if a single quote is located between 2 letters and the word is possessive then
                '   capitalize the letter before the single qoute
                If Mid$(strStringToCap, intLetterToCap, 1) = "'" And _
                    Mid$(strStringToCap, intLetterToCap + 2, 1) <> " " And _
                    Mid$(strStringToCap, intLetterToCap + 1, 1) <> Right$(strStringToCap, 1) Then
                    Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                End If
                
                '   If a graves mark is located between 2 letters and the word is not possessive then
                '   capitalize the letters before and after the graves mark
                '   Or if a graves mark is located between 2 letters and the word is possessive then
                '   capitalize the letter before the graves mark
                If Mid$(strStringToCap, intLetterToCap, 1) = "`" And _
                    Mid$(strStringToCap, intLetterToCap + 2, 1) <> " " And _
                    Mid$(strStringToCap, intLetterToCap + 1, 1) <> Right$(strStringToCap, 1) Then
                    Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                    End If
                    
                    '   If a period is located between 2 letters then save the location of the period
                    '   Capitalize the word following the period and then remove the period
                    'Try this:
                    '       john j. mac.donald
                    '       J.J.VAN.DAMME
                    '       star trek i.v - the voyage home
                    '       STAR TREK, T.N.G
                    '       star trek, t.n.g
                    '       number x.i.i
                    '       i.'ll be back
                    '       a..b normal
                    If Mid$(strStringToCap, intLetterToCap, 1) = "." And _
                        Mid$(strStringToCap, intLetterToCap + 1, 1) <> " " Then
                        intSep = intLetterToCap
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                        GoSub RemovePeriod
                    End If
                    
                    '   Change any single qoute encountered to a graves mark
                    'Try this:
                    '       michael j.o 'herily
                    '       national lampoon 's christmas vacation
                    If Mid$(strStringToCap, intLetterToCap, 1) = "'" Then
                        Mid$(strStringToCap, intLetterToCap, 1) = "`"
                    End If
                    
                    '   Capitalize the words before and after hyphens
                    'Try this:
                    '       john j. jenkle-hymer-smythe
                    If Mid$(strStringToCap, intLetterToCap, 1) = "-" Then
                        ' Capitalize words before and after hyphens
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                    End If
                    
                    '   Capitalize words after a right parenthesis
                    'Try this:
                    '       star trek (episode 13)
                    If Mid$(strStringToCap, intLetterToCap, 1) = "(" Then
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                    End If
                    
                    '   Capitalize words after underscores
                    If Mid$(strStringToCap, intLetterToCap, 1) = "_" Then
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                    End If
                    
                    '   Capitalize words after double quotes
                    If Mid$(strStringToCap, intLetterToCap, 1) = Chr(34) Then
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                    End If
                    
                    '   Capitalize words after slash
                    'Try this:
                    '       This is an action/adventure movie
                    If Mid$(strStringToCap, intLetterToCap, 1) = "/" Then
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                        End If
                        
                    '   Capitalize words preceded by a space
                    If Mid$(strStringToCap, intLetterToCap, 1) = " " Then
                        Mid$(strStringToCap, intLetterToCap + 1, 1) = _
                                                        UCase$(Mid$(strStringToCap, intLetterToCap + 1, 1))
                    End If
                    Next
                End If
            End Select
            Exit Sub
        
RemovePeriod:
    intTextLen = Len(strStringToCap)
    BeforeSep = Left$(strStringToCap, intSep - 1)
    AfterSep = Mid$(strStringToCap, intSep + 1, (intTextLen + 1) - intSep)
    strStringToCap = BeforeSep & AfterSep
    Return
    
ErrHandler:
    MsgBox ("Capitalization Error. Possible invalid character." & Chr$(10) & _
        "A special character may be incorrectly located." & Chr$(10) & _
        "The string cannot end with any character except a letter." & Chr$(10) & _
        "Be sure there is a space between double initials."), vbCritical
End Sub


