'  PRINTING.MAK
'
'  Copyright (C) Jerzy Dziedzic
'  287 N. 9th Street #2A
'  Wheeling, IL 60090
'
'  JerzyD8888@aol.com
'
'
'  This is full code of the sample program showing how to print
'  contents of the Microsoft Grid Control.
'  A compiled version of the program - printing.exe - is also included.
'
'  You have a right to use, modify, reproduce and distribute
'  this file (and/or any modified version) in any way you find useful,
'  provided that you agree that the author - Jerzy Dziedzic - has no
'  warranty, obligation or liability for its contents.

' What it does:
'
' - sets up a printer (common dialog)
'
' - sets up a font used for printing (common dialog)
'
' - prints contents of the grid's cells
'
' - calculates column widths and spacing for both a grid (screen) and
'   a page (printer)
'
' - resizes fonts automatically, if text does not fit the page width
'
' - splits rows between multiple pages, if necessary
'
' - gives a choice of printing (or not) horizontal and vertical
'   gridlines
'
' - gives a choice of creating (or not) a page layout, with headers,
'   footers and a title on every page
'
' - prints page numbering
'
' - allows to abort a printing process by pressing a Cancel button anytime
'
'
' To print contents of a grid, you simply pass the grid (As Control in a
' form of Form.Control) to the PrintGrid Sub.
'
' Flags in the PrintGrid Sub responsible for the page layout, font setup,
' headers and footers are passed as global variables. These variables are
' set in other subs or read from controls on frmMain. Since you will
' probably use just a part of my code, you will have to take care of setting
' and passing these variables in code. If you like, you can change the sub
' and pass necessary variables as parameters instead.
'
' There is one very important limitation - I made an assumption, that text
' strings in all the grid rows are short enough to fit the width of the page.
' This means, that if you have a grid with a large number of columns, or
' very long text strings in grid cells, the program as-it-is will not
' work correctly.
' It is pretty easy, though, to adjust the existing code to print a grid
' of any size. The general idea here would be to print a grid in
' vertical one-page-wide "stripes", instead of as a whole. There are extra
' calculations and parameters necessary, in order to ajust the code this way.
' You will have to calculate a number of columns fitting the page (by checking
' the longest string in every column) and then pass the "stripe" to the
' printing sub (using the starting and ending columns of the "stripe" as new
' parameters). Of course, it would need some further experimenting, but the
' existing code may be a good starting point here.
'
' I tried to add as many comments as I thought was necessary, but if you
' still have problems reading code, e-mail me your questions. I will try to
' answer, if I can.
'
' &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
'
'
' And now the financial part:
'
' I spent many, many (!!!!!) hours writing and testing this program.
' Hopefully my program can save you all these hours and lots of frustration
' as well.
'
' Since You are a programmer, I hope you understand the idea of Shareware.
' If you find my work useful, please send $10 to:
'
' Jerzy Dziedzic
' 287 N. 9th Street #2A
' Wheeling, IL 60090
'
'
' JerzyD8888@aol.com






Option Explicit

Declare Function AbortDoc% Lib "GDI" (ByVal hDC%)
Declare Function SetSysModalWindow% Lib "User" (ByVal hWnd%)
Declare Sub BringWindowToTop Lib "User" (ByVal hWnd%)
Declare Function DeleteDC% Lib "GDI" (ByVal hDC%)

Global PrinterTab() As Long      'X position of the text column in Twips
Global NumberOfCharacters() As String 'the longest string in the column
Global fMaxLengthStringIsBold() As Integer
Global PrintFontName As String
Global PrintFontSize As Integer  'font size after adjustments
Global PrintFontItalic As Integer
Global DialogFontSize As Integer 'font size from a CMDialog
Global PrintFontBold As Integer
Dim TempCellText()
Global FormTitle As String
Global MaxTabSize As Long   'X position of the end of the last text column in Twips
Global Y0 As Integer
Global BottomLineY As Integer
Global NumberOfRows As Integer
Global PersonName As String
Global CompanyName As String


'MsgBox constants:
Global Const MB_OK = 0
Global Const MB_ICONEXCLAMATION = 48
Global Const MB_ICONSTOP = 16
Global Const MB_DEFBUTTON1 = 0
Global Const MB_APPLMODAL = 0

'CMDlog constants
Global Const OFN_READONLY = &H1&
Global Const OFN_OVERWRITEPROMPT = &H2&
Global Const OFN_ALLOWMULTISELECT = &H200&
Global Const OFN_EXTENSIONDIFFERENT = &H400&
Global Const OFN_PATHMUSTEXIST = &H800&
Global Const OFN_FILEMUSTEXIST = &H1000&
Global Const DLG_PRINT = 5
Global Const PD_PRINTSETUP = &H40&
Global Const CDERR_CANCEL = &H7FF3
Global Const DLG_FONT = 4
Global Const CF_BOTH = &H3&
Global Const CF_PRINTERFONTS = &H2&
Global Const CF_NOFACESEL = &H80000
Global Const CF_NOSTYLESEL = &H100000
Global Const CF_NOSIZESEL = &H200000
Global Const PD_NOSELECTION = &H4&
Global Const PD_NOPAGENUMS = &H8&

'global flags
Global fPrintLayout As Integer
Global fPrintVertGridlines As Integer
Global fPrintHorGridlines As Integer
Global fPrintNewForm As Integer
Global fAbortPrinting As Integer

    Sub CenterForm (frm As Form)
	Dim x, y        'New directions for the form
	
	x = (screen.Width - frm.Width) / 2
	y = (screen.Height - frm.Height) / 2
	frm.Move x, y       'Change the location of the form
    End Sub

Sub FillGrid (grid As Control)
'this is a sub filling grid, you do not really need it
'since you fill up your grid somewhere else
'it does not pass any variables

Dim Ix As Integer, I As Integer, MaxCellWidth As Integer


    If NumberOfRows > 200 Then
    'show message to wait
    frmWait.Show
    screen.MousePointer = 11
    DoEvents
    End If
'since I did not know how you fill your grid, I just copied this text all over
grid.rows = NumberOfRows
For Ix = 1 To NumberOfRows - 1
    For I = 0 To grid.Cols - 1
    grid.Col = I
    grid.Row = Ix
Select Case I
    Case 0
    grid.text = "Short text"
    Case 1
    grid.text = "This is a long text entry"
    Case 2
    grid.text = "Text"
    Case 3
    grid.text = "This is an extremely long text entry in the grid"
End Select
'resize cells if necessary

MaxCellWidth = frmMain.TextWidth(grid.text) + 200

    If MaxCellWidth > grid.ColWidth(I) Then
	grid.ColWidth(I) = MaxCellWidth
    End If


Next I
Next Ix

grid.Visible = True
Unload frmWait
screen.MousePointer = 0
End Sub

Sub FindTabSizes (grid As Control)
'It is really a function calculating global arrays:
'NumberOfCharacters()
'fMaxLengthStringIsBold()
'PrinterTab()

Dim NumberOfColumns As Integer
NumberOfColumns = grid.Cols
ReDim PrinterTab(NumberOfColumns)
Dim TempLen As Integer
ReDim NumberOfCharacters(NumberOfColumns)
ReDim fMaxLengthStringIsBold(NumberOfColumns)
Dim Ix As Integer, I As Integer


printer.FontSize = PrintFontSize

If PrintFontName <> "" Then
printer.FontName = PrintFontName
End If

printer.FontSize = PrintFontSize


For I = 0 To NumberOfColumns
PrinterTab(I) = 0
NumberOfCharacters(I) = ""
Next I

For I = 0 To grid.Cols - 1
grid.Col = I

For Ix = 0 To grid.rows - 1
grid.Row = Ix

TempLen = Len(grid.text) + 4
If TempLen > Len(NumberOfCharacters(I)) Then
NumberOfCharacters(I) = grid.text & "    "
If Ix = 0 Then fMaxLengthStringIsBold(I) = True
End If
Next Ix
Next I

PrinterTab(0) = printer.TextWidth("ABC")
For I = 0 To grid.Cols - 1
PrinterTab(I + 1) = PrinterTab(I) + printer.TextWidth(NumberOfCharacters(I))
Next I
MaxTabSize = PrinterTab(grid.Cols)

End Sub

Sub FontSetup ()
Dim Msg$
Dim I As Integer

    On Error GoTo FontSetupError
    frmMain.dlog.CancelError = True
    frmMain.dlog.Flags = CF_BOTH 'CF_PRINTERFONTS
    frmMain.dlog.Action = DLG_FONT        ' Show font setup dialog box
    

PrintFontName = frmMain.dlog.FontName
DialogFontSize = frmMain.dlog.FontSize
PrintFontBold = frmMain.dlog.FontBold
PrintFontItalic = frmMain.dlog.FontItalic


frmMain.Grid1.FontName = PrintFontName
frmMain.Grid1.FontItalic = PrintFontItalic
frmMain.Grid1.FontBold = PrintFontBold
frmMain.FontSize = DialogFontSize
frmMain.Grid1.FontSize = DialogFontSize
If frmMain.Grid1.Visible = True Then FillGrid frmMain.Grid1


FontSetupExit:
    Exit Sub

FontSetupError:
    If Err <> CDERR_CANCEL Then         ' If user didn't select CANCEL
	Beep
	Msg$ = "Error" & Str$(Err) & ": " & Error$
	MsgBox Msg$, MB_ICONEXCLAMATION, "Error"
    End If
    Resume FontSetupExit

End Sub

Sub PrinterSetup ()
Dim Msg$

    On Error GoTo FilePrintSetupError

    frmMain.dlog.Flags = PD_PRINTSETUP     ' Just enable print setup
    frmMain.dlog.Action = DLG_PRINT        ' Show printer setup dialog box

PrintSetupExit:
    Exit Sub

FilePrintSetupError:
    If Err <> CDERR_CANCEL Then         ' If user didn't select CANCEL
	Beep
	Msg$ = "Error" & Str$(Err) & ": " & Error$
	MsgBox Msg$, MB_ICONEXCLAMATION, "Error"
    End If
    Resume PrintSetupExit

End Sub

Sub PrintGrid (grid As Control)
'These are variables passed to this Sub:
'
'As parameter - Grid, which is a name of the grid
'in form of Formname.Controlname
'
'As global variables:

'frmMain - which is the name of the form with a grid
'PrintFontName - comes from Sub FontSetup
'DialogFontSize - comes from Sub FontSetup
'PrintFontSize - it is set here but may be reset through sub ResizeFont
'fPrintLayout - comes from the form's radio button
'fPrintHorGridlines - comes from the form's check box
'fPrintVertGridlines  - comes from the form's check box
'BottomLineY and Y0 - they may be set here or in the PrintPageLayout sub
'PrinterTab() - this array comes from the FindTabSizes sub

'if you cannot find what is what, please e-mail me.
'
'jerryd8888@aol.com


    ' Declare local variables.
    Dim NumberOfColumns As Integer
    Dim x1 As Integer, y1 As Integer
    Dim Ix As Integer, I As Integer
    NumberOfColumns = grid.Cols
    ReDim TempCellText(NumberOfColumns)
    Dim MaximumWidth As Integer, LastLineY As Integer

DoEvents
    ' Change the mouse pointer to the hourglass.
    frmMain.MousePointer = 11
    fAbortPrinting = False
    
    PrnWait.Show
    BringWindowToTop PrnWait.hWnd
    PrnWait.Refresh
    x1 = SetSysModalWindow(PrnWait.hWnd)

    fPrintNewForm = True 'flag for the layout printing
    
    'You can put here any title you want
    FormTitle = Trim$(frmMain.Caption)
    

    If PrintFontName <> "" Then
    'if CMDialog was used
       printer.FontName = PrintFontName
    End If
    
    If DialogFontSize = 0 Then
	PrintFontSize = 10
    Else
	'if CMDialog was used
	If DialogFontSize < 16 Then
	PrintFontSize = DialogFontSize
	Else
	'too large font does not make sense here
	'but if you like, you can always change this limit
	PrintFontSize = 16
	End If
    End If

    ' find tab sizes in the current grid
    DoEvents
    If fAbortPrinting = True Then GoTo PrintingCancelled
    FindTabSizes grid

    'change the font size, or page orientation, if necessary
    DoEvents
    If fAbortPrinting = True Then GoTo PrintingCancelled

    ResizeFont grid

' Draw a box that represents the outline of the form.
    printer.DrawWidth = 2
    DoEvents
    If fAbortPrinting = True Then GoTo PrintingCancelled
    
    If fPrintLayout = True Then
    PrintPageLayout
    MaximumWidth = printer.Width * .94 - 200
    Else
    MaximumWidth = PrinterTab(grid.Cols) - 200 - 4 * printer.TwipsPerPixelX
    If fPrintHorGridlines = True Then printer.Line (200, 0)-Step(MaximumWidth, 0)
    End If
     
For Ix = 0 To grid.rows - 1
    
    If printer.CurrentY > printer.Height * .92 - printer.TextHeight("TEST") Then
    'first check if printing was not cancelled
    DoEvents
    If fAbortPrinting = True Then GoTo PrintingCancelled
    'draw vertical lines
    If fPrintVertGridlines = True Then
    If fPrintLayout = False Then
    BottomLineY = LastLineY
    Y0 = 0
    printer.Line (200, Y0)-(200, BottomLineY)
    For I = 1 To grid.Cols
    printer.Line (PrinterTab(I) - 4 * printer.TwipsPerPixelX, Y0)-(PrinterTab(I) - 4 * printer.TwipsPerPixelX, BottomLineY)
    Next I
    End If

    For I = 1 To grid.Cols - 1
    printer.Line (PrinterTab(I) - 4 * printer.TwipsPerPixelX, Y0)-(PrinterTab(I) - 4 * printer.TwipsPerPixelX, BottomLineY)
    Next I
    End If

    
    'print new page layout
    DoEvents
    If fAbortPrinting = True Then GoTo PrintingCancelled

    printer.NewPage
    If fPrintLayout = True Then
    PrintPageLayout
    Else
    If fPrintHorGridlines = True Then printer.Line (200, 0)-Step(MaximumWidth, 0)
    End If
    printer.FontSize = PrintFontSize
    printer.FontBold = True
    printer.CurrentX = 50
    x1 = printer.CurrentX
    printer.CurrentY = printer.CurrentY + 350
    y1 = printer.CurrentY
	grid.Row = 0
	For I = 0 To grid.Cols - 1
	grid.Col = I
	TempCellText(I) = grid.text
	printer.CurrentX = x1 + PrinterTab(I)
	printer.CurrentY = y1
	  'print column headers
	printer.Print TempCellText(I)
	Next I
	printer.Print
	If fPrintHorGridlines = True Then printer.Line (200, printer.CurrentY)-Step(MaximumWidth, 0) 'underline headers
     printer.CurrentX = 220
     printer.CurrentY = printer.CurrentY + printer.TwipsPerPixelY * 2
     printer.FontBold = PrintFontBold
    
    End If

    grid.Row = Ix
	For I = 0 To grid.Cols - 1

	'get text from the current row in the grid
	grid.Col = I
	TempCellText(I) = grid.text
	Next I

	If Ix = 0 Then
	'make column headers bold
	printer.FontBold = True
	Else
	printer.FontBold = PrintFontBold
	'and the rest normal or whatever the user wants
	End If

    printer.FontSize = PrintFontSize
    printer.CurrentX = 50
    x1 = printer.CurrentX
    If Ix = 0 Then
    printer.CurrentY = printer.CurrentY + 350
    Else
    printer.CurrentY = printer.CurrentY + 2 * printer.TwipsPerPixelY
    End If
    y1 = printer.CurrentY
	grid.Row = Ix
	For I = 0 To grid.Cols - 1
	grid.Col = I
	TempCellText(I) = grid.text
	printer.CurrentX = x1 + PrinterTab(I)
	printer.CurrentY = y1
	printer.Print TempCellText(I)
	Next I
	 
	 If Ix = 0 Then printer.Print
	 If fPrintHorGridlines = True Then
	 printer.Line (200, printer.CurrentY)-Step(MaximumWidth, 0)
	 LastLineY = printer.CurrentY
	 Else
	 printer.Print
	 End If
	 Next Ix

    If fPrintVertGridlines = True Then
    'draw vertical lines
    If fPrintLayout = False Then
    BottomLineY = LastLineY
    Y0 = 0
    printer.Line (200, Y0)-(200, BottomLineY)
    For I = 1 To grid.Cols
    printer.Line (PrinterTab(I) - 4 * printer.TwipsPerPixelX, BottomLineY)-(PrinterTab(I) - 4 * printer.TwipsPerPixelX, Y0)
    Next I
    Else
    For I = 1 To grid.Cols - 1
    printer.Line (PrinterTab(I) - 4 * printer.TwipsPerPixelX, BottomLineY)-(PrinterTab(I) - 4 * printer.TwipsPerPixelX, Y0)
    Next I
    End If
    End If

    DoEvents
    If fAbortPrinting = True Then GoTo PrintingCancelled

    ' Send contents of Printer object to printer.
    printer.EndDoc
    
' Change the mouse pointer back to default.
    frmMain.MousePointer = 0
    Unload PrnWait
Exit Sub

PrintingCancelled:
screen.MousePointer = 0
frmMain.MousePointer = 0
x1 = AbortDoc(printer.hDC)
On Error Resume Next
printer.EndDoc

Exit Sub


End Sub

Sub PrintPageLayout ()
Static PageCount As Integer
Dim PersonalInfo As String


    'page number
    If fPrintNewForm = True Then PageCount = 1
    
    ' Print the program's header.
    printer.FontSize = 6
    printer.FontBold = False
    printer.CurrentX = 200
    printer.CurrentY = 295 - printer.TextHeight("JUST TESTING")
    printer.Print "Printing Demo"
    printer.CurrentX = (printer.Width * .94 - printer.TextWidth(Chr(169) & " Jerzy Dziedzic, frmMain, Inc."))
    printer.CurrentY = 295 - printer.TextHeight("JUST TESTING")
    printer.Print Chr(169) & " Jerzy Dziedzic, Practical Software, Inc."
    
    printer.FontSize = 8
    printer.FontBold = False
    BottomLineY = printer.Height * .94 - printer.TextHeight("JUST TESTING") - printer.TwipsPerPixelY
     'draw a border
    printer.Line (200, 300)-Step(printer.Width * .94 - 200, BottomLineY - 300), , B


    ' Print footer - personal information.
    printer.FontSize = 8
    printer.FontBold = False
    printer.CurrentX = 215
    printer.CurrentY = BottomLineY + printer.TwipsPerPixelY
    PersonalInfo = ""
    If PersonName <> "" Then PersonalInfo = PersonName
    If CompanyName <> "" Then PersonalInfo = PersonalInfo & ", " & CompanyName & "   "
    printer.Print PersonalInfo & Format$(Now, "Long Date")
    ' and the page number
    printer.FontSize = 8
    printer.FontBold = False
    printer.CurrentX = (printer.Width * .94 - printer.TextWidth(" Page  " & PageCount))
    printer.CurrentY = BottomLineY + printer.TwipsPerPixelY
    printer.Print "Page  " & PageCount


    ' Print Title bar
    printer.FontSize = Int(2 * PrintFontSize)
    printer.FontBold = True
    Y0 = printer.TextHeight(FormTitle) * 2 + 300
    printer.Line (200, Y0)-Step(printer.Width * .94 - 200, 0)
    printer.CurrentX = (printer.Width - printer.TextWidth(FormTitle) - 200) / 2
    printer.CurrentY = printer.TextHeight(FormTitle) / 2 + 300
    'instead of FormTitle you can put here any text you want
    printer.Print FormTitle
    PageCount = PageCount + 1
    'reset the font to a default value
    printer.FontSize = PrintFontSize
    printer.FontBold = PrintFontBold
    fPrintNewForm = False  'set page numbering flag
End Sub

Sub ResizeFont (grid As Control)
'It is really a function re-calculating an array PrinterTab()
Dim PrinterPageWidth As Integer, I As Integer


PrinterPageWidth = printer.Width * .94 - 200
'MaxTabSize is passed here as global variable from FindTabSizes sub
'It may be reset here, if font is too large
If PrinterPageWidth > MaxTabSize Then Exit Sub

Do Until PrinterPageWidth >= MaxTabSize
'Font variables are passed as global from FontSetup sub
'and from PrintGrid sub
PrintFontSize = PrintFontSize - 1
printer.FontSize = PrintFontSize

If PrintFontName <> "" Then
printer.FontName = PrintFontName
End If

printer.FontSize = PrintFontSize

PrinterTab(0) = printer.TextWidth("AB")
For I = 0 To grid.Cols - 1
If fMaxLengthStringIsBold(I) = True Then
printer.FontBold = True
Else
printer.FontBold = PrintFontBold
End If

PrinterTab(I + 1) = PrinterTab(I) + printer.TextWidth(NumberOfCharacters(I))
Next I
MaxTabSize = PrinterTab(grid.Cols)

Loop

End Sub

