Attribute VB_Name = "msharedCommon"
Option Explicit

'This module contains procedures which are common to
'all applications

Private Type OsVersionInfo
    dwOSVersionInfoSize As Long
    dwMajorVersion As Long
    dwMinorVersion As Long
    dwBuildNumber As Long
    dwPlatformId As Long
    szCSDVersion As String * 128
End Type

Private Declare Function GetVersionEx Lib "kernel32" Alias "GetVersionExA" (lpVersionInformation As OsVersionInfo) As Long
'  dwPlatformId defines:
Private Const VER_PLATFORM_WIN32s = 0
Private Const VER_PLATFORM_WIN32_WINDOWS = 1
Private Const VER_PLATFORM_WIN32_NT = 2

Public Type WINVERSIONINFO
    Platform As Integer
    Major As Integer
    Minor As Integer
    Build As Integer
    AdditionalInfo As String
End Type

'Application-defined constants for the platform
Public Const WVI_PLATFORM_WIN95 = 1
Public Const WVI_PLATFORM_WIN95_OSR2 = 2
Public Const WVI_PLATFORM_WIN98 = 3
Public Const WVI_PLATFORM_WIN_NT = 4
Public Const WVI_PLATFORM_WIN32S = 5

Private Declare Function GetWindowsDirectory Lib "kernel32" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Private Declare Function GetSystemDirectory Lib "kernel32" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long

Public gsWinDir As String
Public gsSysDir As String

Function FileOrDirExists(Optional ByVal varFileName As Variant, Optional ByVal varDirectory As Variant) As Boolean

'This function can be used to determine if either a file
'or a directory exists.

'Both arguments are optional but at least one of them MUST be passed.

'If only a filename is passed, the functions checks if that file
'exists in the specified path.  If no path is included as part of the
'the filename, the directory returned by CurDir is assumed.
'Ex.  bExists = FileOrDirExists("c:\windows\notepad.exe")

'If only a directory is passed, the function checks if that directory exists
'Ex.  bExists = FileOrDirExists(, "c:\windows\system")

'If both a filename and directory are passed, the function checks if the
'specified file exists in the specified directory.
'Ex.  bExists = FileOrDirExists("notepad.exe", "c:\windows")


'NOTES:
'   If a directory is passed, you should NOT include a path as part of the
'   filename.  However, if both are specified AND a path is included
'   as part of the filename, the varDirectory argument will be ignored.
'   Ex.  bExists = FileOrDirExists("c:\windows\notepad.exe", "c:\windows")

'   A trailing backslash for the directory can be included or omitted.

'   The function returns:
'   True (-1) if the file or directory exists
'   False (0) if the file or directory does not exist

'   Variants were used to maintain compatibility with VB4.  VB5 allows
'   optional arguments to be of any data type, but VB4 only allows
'   optional arguments to be variants.

If IsMissing(varDirectory) Then
    'Only a file name was passed.
    If Len(Dir$(varFileName)) Then FileOrDirExists = True
Else
    'A directory was passed
    'Append a backslash to the directory, if necessary.
    If Right$(varDirectory, 1) <> "\" Then varDirectory = varDirectory & "\"
    
    If IsMissing(varFileName) Then
        'Directory was passed, but not a filename, so determine if
        'that directory exists
        If Len(Dir$(varDirectory, vbDirectory)) Then FileOrDirExists = True
    Else
        'Both a directory and a filename were passed.  If a path was
        'included as part of the filename, disregard varDirectory.
        If InStr("\", varFileName) Then
            If Len(Dir$(varFileName)) Then FileOrDirExists = True
        Else
            If Len(Dir$(varDirectory & varFileName)) Then FileOrDirExists = True
        End If
    End If
End If

End Function

Public Function RemoveFileName(sFileName As String) As String

'Removes the file name from the passed string and returns
'just the path.

Dim nExists%, nPosition%, sTemp$

nExists = InStr(sFileName, "\")

'loop until the last backslash has been found
Do While nExists
   nPosition = nExists
   nExists = InStr(nPosition + 1, sFileName, "\")
Loop

If nPosition > 0 Then
    'Assign everything left of the last backslash
    sTemp = Left$(sFileName, nPosition - 1)
End If

RemoveFileName = sTemp

End Function
 
Public Sub CenterForm(Frm As Form)
    
Frm.Move (Screen.Width - Frm.Width) \ 2, (Screen.Height - Frm.Height) \ 2

End Sub



Public Function GetHiByte(Word As Integer) As Integer

GetHiByte = Word \ &H100 And &HFF&

End Function

Public Function GetHiWord(dWord As Long) As Integer

GetHiWord = dWord \ &H10000 And &HFFFF&

End Function

Public Function GetLoByte(Word As Integer) As Integer

GetLoByte = Word And &HFF&

End Function

Public Function GetLoWord(dWord As Long) As Integer

GetLoWord = dWord And &HFFFF&

End Function


Public Function GetWinDir() As String

Dim sBuffer As String
Dim nBuffer As Long
Dim nBytes As Long

sBuffer = Space$(255)
nBuffer = Len(sBuffer)
nBytes = GetWindowsDirectory(sBuffer, nBuffer)
GetWinDir = Left$(sBuffer, nBytes)

End Function

Public Function GetWinSysDir() As String

Dim sBuffer As String
Dim nBuffer As Long
Dim nBytes As Long

sBuffer = Space$(255)
nBuffer = Len(sBuffer)
nBytes = GetSystemDirectory(sBuffer, nBuffer)
GetWinSysDir = Left$(sBuffer, nBytes)

End Function

Public Function RemoveNulls(ByVal sTemp As String) As String

'Removes terminating null characters from the passed string

RemoveNulls = Left$(sTemp, InStr(sTemp, vbNullChar) - 1)

End Function



Public Function GetHiLoByte(ByVal Word As Integer, HiByte As Integer, LoByte As Integer) As Boolean

HiByte = Word \ &H100 And &HFF&
LoByte = Word And &HFF&

GetHiLoByte = True

End Function

Public Function GetHiLoWord(ByVal dWord As Long, LoWord As Long, HiWord As Long) As Boolean

' This is the LOWORD of dWord:
LoWord = dWord And &HFFFF&

' This is the HIWORD of dWord:
HiWord = dWord \ &H10000 And &HFFFF&

GetHiLoWord = True

End Function

Public Function RemovePath(sFullFileName As String) As String

'Removes the path from the passed string and returns just
'the filename.

Dim nExists As Integer
Dim nPosition As Integer
Dim sTemp As String

nExists = InStr(sFullFileName, "\")

'loop until the last backslash has been found
Do While nExists
   nPosition = nExists
   nExists = InStr(nPosition + 1, sFullFileName, "\")
Loop

If nPosition > 0 Then
    sTemp = Mid$(sFullFileName, nPosition + 1)
End If

RemovePath = sTemp

End Function
Public Function GetWinVerInfo(wvi As WINVERSIONINFO) As Boolean

Dim OSVer As OsVersionInfo
Dim lResult As Long

OSVer.dwOSVersionInfoSize = Len(OSVer)
lResult = GetVersionEx(OSVer)

If lResult Then
    GetWinVerInfo = True
Else
    GetWinVerInfo = False
    Exit Function
End If

Select Case OSVer.dwPlatformId
    Case VER_PLATFORM_WIN32s
        wvi.Platform = WVI_PLATFORM_WIN32S
    Case VER_PLATFORM_WIN32_WINDOWS
        Select Case OSVer.dwMinorVersion
            Case 0
                'If the low word of the build number is >= 1111, then OSR2 is installed
                If GetLoWord(OSVer.dwBuildNumber) = 1111 Then
                    wvi.Platform = WVI_PLATFORM_WIN95_OSR2
                Else
                    wvi.Platform = WVI_PLATFORM_WIN95
                End If
            Case 10:
                wvi.Platform = WVI_PLATFORM_WIN98
        End Select
    Case VER_PLATFORM_WIN32_NT
        wvi.Platform = WVI_PLATFORM_WIN_NT
End Select

wvi.Major = OSVer.dwMajorVersion
wvi.Minor = OSVer.dwMinorVersion
wvi.Build = GetLoWord(OSVer.dwBuildNumber)
wvi.AdditionalInfo = RemoveNulls(OSVer.szCSDVersion)

End Function

