Attribute VB_Name = "msharedRegistry"
'Only 4 of the functions in this module are public.  These are the
'CreateRegKey, DeleteRegKey, GetRegStringValue, and WriteRegStringValue
'functions.  The 3 remaining user-defined functions and the Registry
'API functions are private.  There should be no need to have to call
'these from outside this module unless you wanted to re-write it
'and do things differently from what I have written, which you
'are perfectly welcome to do.  As is, this module only supports
'querying string values; however, the constants needed to query other
'types of values are included.  It should not be too difficult to add
'support for these other types.   This module is NOT intended to be used
'for an editor-type program.  All Registry API functions (except
'less-functional counter-parts), constants, etc., have been declared
'for completeness, but they are not all used.

Option Explicit

'Required for RegEnumKey and RegQueryInfoKey
Private Type FILETIME
    lLowDateTime    As Long
    lHighDateTime   As Long
End Type


'*****IF THE FOLLOWING ERROR IS RECEIVED (PARTCULARLY UNDER nt 4.0)
'*****"THE PARAMETER IS INCORRECT" (API ERROR 87), THE BELOW DECLARATIONS
'*****MIGHT NEED TO BE CHANGED TO INCLUDE "BYVAL" BEFORE CERTAIN
'*****PARAMETERS.

Private Declare Function RegOpenKeyEx& Lib "advapi32.dll" _
Alias "RegOpenKeyExA" (ByVal hKey&, ByVal lpszSubKey$, _
dwOptions&, ByVal samDesired&, lpHKey&)

Private Declare Function RegOpenKey Lib "advapi32.dll" _
Alias "RegOpenKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, _
phkResult As Long) As Long

Private Declare Function RegCreateKey& Lib "advapi32" Alias _
"RegCreateKeyA" (ByVal hKey&, ByVal lpszSubKey$, phkResult&)

Private Declare Function RegCreateKeyEx& Lib "advapi32.dll" Alias _
"RegCreateKeyExA" (ByVal hKey&, ByVal lpSubKey$, ByVal Reserved&, _
ByVal lpClass$, ByVal dwOptions&, ByVal samDesired&, _
lpSecurityAttributes&, phkResult&, lpdwDisposition&)

Private Declare Function RegDeleteKey& Lib "advapi32" Alias _
"RegDeleteKeyA" (ByVal hKey As Long, ByVal lpszSubKey As String)

Private Declare Function RegCloseKey& Lib "advapi32.dll" _
(ByVal hKey&)

Private Declare Function RegQueryValueEx& Lib "advapi32.dll" _
Alias "RegQueryValueExA" (ByVal hKey&, ByVal lpszValueName$, _
ByVal lpdwRes&, lpdwType&, ByVal lpDataBuff$, nSize&)

Private Declare Function RegQueryValue Lib "advapi32.dll" _
Alias "RegQueryValueA" (ByVal hKey As Long, ByVal lpSubKey As String, _
ByVal lpValue As String, lpcbValue As Long) As Long


Private Declare Function RegSetValueEx& Lib "advapi32.dll" Alias _
"RegSetValueExA" (ByVal hKey&, ByVal lpszValueName$, ByVal dwRes&, _
ByVal dwType&, lpDataBuff As Any, ByVal nSize&)

Private Declare Function RegConnectRegistry& Lib "advapi32.dll" _
(ByVal lpMachineName$, ByVal hKey&, phkResult&)

Private Declare Function RegFlushKey& Lib "advapi32.dll" (ByVal hKey&)

Private Declare Function RegEnumKeyEx& Lib "advapi32.dll" Alias _
"RegEnumKeyExA" (ByVal hKey&, ByVal dwIndex&, ByVal lpName$, _
lpcbName&, ByVal lpReserved&, ByVal lpClass$, lpcbClass&, _
lpftLastWriteTime As FILETIME)

Private Declare Function RegEnumValue& Lib "advapi32.dll" Alias _
"RegEnumValueA" (ByVal hKey&, ByVal dwIndex&, ByVal lpName$, _
lpcbName&, ByVal lpReserved&, lpdwType&, lpValue As Any, lpcbValue&)

Private Declare Function RegQueryInfoKey& Lib "advapi32.dll" Alias _
"RegQueryInfoKeyA" (ByVal hKey&, ByVal lpClass$, lpcbClass&, _
ByVal lpReserved&, lpcSubKeys&, lpcbMaxSubKeyLen&, lpcbMaxClassLen&, _
lpcValues&, lpcbMaxValueNameLen&, lpcbMaxValueLen&, _
lpcbSecurityDescriptor&, lpftLastWriteTime As FILETIME)

'Error codes returned from Registry functions.
Const ERROR_SUCCESS = 0&
Const ERROR_BADDB = 1009&
Const ERROR_BADKEY = 1010&
Const ERROR_CANTOPEN = 1011&
Const ERROR_CANTREAD = 1012&
Const ERROR_CANTWRITE = 1013&
Const ERROR_OUTOFMEMORY = 14&
Const ERROR_INVALID_PARAMETER = 87&
Const ERROR_ACCESS_DENIED = 5&
Const ERROR_NO_MORE_ITEMS = 259&
Const ERROR_MORE_DATA = 234&

'Registry value types
Const REG_NONE = 0&                        ' No value type
Const REG_SZ = 1&                          ' Unicode nul terminated string
Const REG_EXPAND_SZ = 2&                   ' Unicode nul terminated string
                                           ' (with environment variable references)
Const REG_BINARY = 3&                      ' Free form binary
Const REG_DWORD = 4&                       ' 32-bit number
Const REG_DWORD_LITTLE_ENDIAN = 4&         ' 32-bit number (same as REG_DWORD)
Const REG_DWORD_BIG_ENDIAN = 5&            ' 32-bit number
Const REG_LINK = 6&                        ' Symbolic Link (unicode)
Const REG_MULTI_SZ = 7&                    ' Multiple Unicode strings
Const REG_RESOURCE_LIST = 8&               ' Resource list in the resource map
Const REG_FULL_RESOURCE_DESCRIPTOR = 9&    ' Resource list in the hardware description
Const REG_RESOURCE_REQUIREMENTS_LIST = 10&

'Registry Read/Write permissions:
Const KEY_QUERY_VALUE = &H1&
Const KEY_SET_VALUE = &H2&
Const KEY_CREATE_SUB_KEY = &H4&
Const KEY_ENUMERATE_SUB_KEYS = &H8&
Const KEY_NOTIFY = &H10&
Const KEY_CREATE_LINK = &H20&
Const READ_CONTROL = &H20000
Const WRITE_DAC = &H40000
Const WRITE_OWNER = &H80000
Const SYNCHRONIZE = &H100000
Const STANDARD_RIGHTS_REQUIRED = &HF0000
Const STANDARD_RIGHTS_READ = READ_CONTROL
Const STANDARD_RIGHTS_WRITE = READ_CONTROL
Const STANDARD_RIGHTS_EXECUTE = READ_CONTROL
Const KEY_READ = STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or _
KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY
Const KEY_WRITE = STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or _
KEY_CREATE_SUB_KEY
Const KEY_EXECUTE = KEY_READ

Public Const REG_ERROR = "REGISTRY_ERROR"


Public Function DeleteRegKey(ByVal sKeyName As String) As Boolean

'This functions deletes the passed key.  It returns True if the
'key is deleted and False if an error occurred or the key didn't
'exist to begin with.  No error message is displayed (unless there
'is a parsing error) because there is really no point in it.  The only
'likely error would be because the key does not exist and we wanted
'to delete it anyway.

Dim hKey As Long, lRtn As Long
Dim hMainKey As Long

DeleteRegKey = False

hMainKey = GetHKEYHandle(sKeyName)

If hMainKey Then
    'Open the key
    lRtn = RegOpenKeyEx(hMainKey, sKeyName, 0&, KEY_WRITE, hKey)
    If lRtn = ERROR_SUCCESS Then
        lRtn = RegDeleteKey(hKey, sKeyName)
        lRtn = RegCloseKey(hKey)
        DeleteRegKey = True
    End If
End If


End Function

Private Function GetMainKeyHandle(ByVal sMainKey As String) As Long

'Returns the handle of the main key (the constants below)

Const HKEY_CLASSES_ROOT = &H80000000
Const HKEY_CURRENT_USER = &H80000001
Const HKEY_LOCAL_MACHINE = &H80000002
Const HKEY_USERS = &H80000003
Const HKEY_PERFORMANCE_DATA = &H80000004
Const HKEY_CURRENT_CONFIG = &H80000005
Const HKEY_DYN_DATA = &H80000006
   
Select Case sMainKey
    Case "HKEY_CLASSES_ROOT"
        GetMainKeyHandle = HKEY_CLASSES_ROOT
    Case "HKEY_CURRENT_USER"
        GetMainKeyHandle = HKEY_CURRENT_USER
    Case "HKEY_LOCAL_MACHINE"
        GetMainKeyHandle = HKEY_LOCAL_MACHINE
    Case "HKEY_USERS"
        GetMainKeyHandle = HKEY_USERS
    Case "HKEY_PERFORMANCE_DATA"
        GetMainKeyHandle = HKEY_PERFORMANCE_DATA
    Case "HKEY_CURRENT_CONFIG"
        GetMainKeyHandle = HKEY_CURRENT_CONFIG
    Case "HKEY_DYN_DATA"
        GetMainKeyHandle = HKEY_DYN_DATA
End Select

End Function

Private Function GetRegError(ByVal lErrorCode As Long) As String
    
'This function returns the error string associated with error
'codes returned by the Registry API functions

Select Case lErrorCode
    Case 1009, 1015
        'As Doomsday* would say "We're in trouble now!"
        GetRegError = "The Registry Database is corrupt!"
    Case 2, 1010
        GetRegError = "Bad Key Name"
    Case 1011
        GetRegError = "Can't Open Key"
    Case 4, 1012
        GetRegError = "Can't Read Key"
    Case 5
        GetRegError = "Access to this key is denied"
    Case 1013
        GetRegError = "Can't Write Key"
    Case 8, 14
        GetRegError = "Out of memory"
    Case 87
        GetRegError = "Invalid Parameter"
    Case 234
        GetRegError = "There is more data than the buffer has been allocated to hold."
    Case Else
        GetRegError = "Undefined Error Code:  " & Str$(lErrorCode)
End Select

'* - Haven't you ever played Wing Commander?  :)
End Function

Private Function GetHKEYHandle(ByRef sKeyName As String) As Long
    
'This sub parses the passed keyname, separates the main HKEY part from
'the rest of the keyname, and returns the main key handle.  Every
'Registry API function requires this handle for the HKEY.  The rest
'of the keyname is passed to the API reg functions as another argument;
'therefore, we'll separate them in this function.  The value returned
'in the sKeyName argument will contain the part of the keyname that
'the API reg functions require.  This is the reason the argument is
'passed by reference instead of by value.

'Pass the full path to the key in this format:
'HKEY_CURRENT_USER\Control Panel\Desktop

Dim lKeyHandle As Long
Dim nBackSlash As Integer

'A simple check to make sure the key was passed correctly
If Left(sKeyName, 5) <> "HKEY_" Or Right(sKeyName, 1) = "\" Then
    MsgBox "Incorrect Format:" & vbCrLf & vbCrLf & sKeyName
    Exit Function
End If

nBackSlash = InStr(sKeyName, "\")
If nBackSlash = 0 Then
    'Only the main key was specified
    lKeyHandle = GetMainKeyHandle(sKeyName)
    sKeyName = ""
Else
    'Get the handle to the main key
    lKeyHandle = GetMainKeyHandle(Left$(sKeyName, nBackSlash - 1))
    'Strip the HKEY part of the keyname away
    sKeyName = Mid$(sKeyName, nBackSlash + 1)
End If

'Make sure the handle is valid (just another check)
If lKeyHandle < &H80000000 Or lKeyHandle > &H80000006 Then
    MsgBox "Not a valid HKEY handle", vbCritical
    lKeyHandle = 0
End If

GetHKEYHandle = lKeyHandle

End Function
Public Function GetRegStringValue(ByVal sKeyName As String, ByVal sSetting As String, ByRef sValue As String, Optional bNoErrMessage As Boolean = True) As Boolean

Dim hKey As Long, hMainKey As Long
Dim lRtn As Long, sBuffer As String
Dim lBufferSize As Long, lType As Long
Dim sErrMsg As String

lType = REG_SZ
    
hMainKey = GetHKEYHandle(sKeyName)

If hMainKey Then
    'Open the key
    lRtn = RegOpenKeyEx(hMainKey, sKeyName, 0&, KEY_READ, hKey)
    If lRtn = ERROR_SUCCESS Then
        'Query for the data
        sBuffer = Space(255)
        lBufferSize = Len(sBuffer)
        lRtn = RegQueryValueEx(hKey, sSetting, 0&, lType, sBuffer, lBufferSize)
        If lRtn = ERROR_SUCCESS Then
            'Remove spaces and eliminate the terminating character
            sBuffer = Trim(sBuffer)
            sValue = Left(sBuffer, Len(sBuffer) - 1)
            GetRegStringValue = True
        Else
            If Not bNoErrMessage Then
                sErrMsg = GetRegError(lRtn)
                MsgBox sErrMsg, vbCritical, "Registry Query Error"
            End If
            GetRegStringValue = False
        End If
    Else
        If Not bNoErrMessage Then
            sErrMsg = GetRegError(lRtn)
            MsgBox sErrMsg, vbCritical, "Registry Open Error"
        End If
    End If
End If

End Function

Public Function WriteRegStringValue(ByVal sKeyName As String, sSetting As String, sValue As String, Optional bNoErrMessage As Boolean = True) As Boolean

'Returns True if successful; otherwise False

Dim hKey As Long, hMainKey As Long
Dim lRtn As Long, lDataSize As Long
Dim lType As Long, sErrMsg As String

WriteRegStringValue = False

lType = REG_SZ

hMainKey = GetHKEYHandle(sKeyName)

If hMainKey Then
    'Open the key
    lRtn = RegOpenKeyEx(hMainKey, sKeyName, 0&, KEY_WRITE, hKey)
    If lRtn = ERROR_SUCCESS Then
        'Write the value
        lDataSize = Len(sSetting)
        lRtn = RegSetValueEx(hKey, sSetting, 0&, lType, ByVal sValue, lDataSize)
        If lRtn = ERROR_SUCCESS Then
            WriteRegStringValue = True
        Else
            If Not bNoErrMessage Then
                sErrMsg = GetRegError(lRtn)
                MsgBox sErrMsg, vbCritical, "Registry Write Error"
            End If
            WriteRegStringValue = False
        End If
        lRtn = RegCloseKey(hKey)
    Else
        If Not bNoErrMessage Then
            sErrMsg = GetRegError(lRtn)
            MsgBox sErrMsg, vbCritical, "Registry Open Error"
        End If
        WriteRegStringValue = False
    End If
End If

End Function

Public Function CreateRegKey(ByVal sNewKey As String, Optional ByVal bNoErrMessage As Boolean = True) As Boolean

' PURPOSE:  Creates a new key in the Registry.
' RETURNS:  True if successful; False if an error occurred.
' ARGUMENTS:
'   sNewKey:  Required; String expression specifying the key to create.
'             The complete path for the key to be created must be passed.
'             I.E.  HKEY_CURRENT_USER\Software\MyCompany\MyApplication
'   bNoErrMessage:  Optional; Boolean expression specifying if error
'                     messages are displayed in a message box.  If not
'                     specified, error messages are NOT displayed (default)
' NOTES:
'      Creating a key also opens it.  The rest of the functions are
'      written with the assumption that the key is closed.
'      While you could change this to return the
'      handle of the created key, you would also have to modify the
'      rest of the functions to pass them the handle.  Besides, it's
'      much safer to NOT have the keys open any longer than absolutely
'      necesssary.  If keys are open and Windows crashes, there's a power
'      outage, etc., the entire Registry could get corrupted.


Dim hMainKey As Long, hKey As Long
Dim sErrMsg As String, lRtn As Long

hMainKey = GetHKEYHandle(sNewKey)

'Create the subkey if we have a handle to the main key
If hMainKey Then
    lRtn = RegCreateKey(hMainKey, sNewKey, hKey)
    If lRtn = ERROR_SUCCESS Then
        'key successfully created.  Close it
        lRtn = RegCloseKey(hKey)
        CreateRegKey = True
    Else
        'Key was not created.  Check if we're showing error messages
        If Not bNoErrMessage Then
            sErrMsg = GetRegError(lRtn)
            MsgBox sErrMsg, vbCritical
        End If
        CreateRegKey = False
    End If
End If

End Function
