VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsSound"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Declare Function sndPlaySound Lib "WINMM.DLL" Alias "sndPlaySoundA" _
(ByVal lpszSoundName As Any, ByVal uFlags As Long) As Long

Enum SoundFlagConstants
'Constants for the uFlags argument
    ssSndSync = &H0      ' Control does not return until sound completes
    ssSndAsync = &H1     ' Control returns immediately after being playback
    ssSndNoDefault = &H2 ' Don't play default sound if lpszSoundName is invalid
    ssSndMemory = &H4    ' lpszSoundName points to a memory file
    ssSndLoop = &H8      ' Continously loops the sound.  SND_ASYNC MUST ALSO BE SPECIFIED, otherwise control will never return
    ssSndNoStop = &H10   ' Do not stop playback of a sound already playing
End Enum

'This flag determines if sounds are on or off.  This is set to True or False
'by the class's Enabled property.  This is useful so you can provide the user with
'an option to play sounds or to not play sounds.  This flag can be overridden when
'invoking the BeginPlaySound method.  See comments in that procedure for details.
Dim mbPlaySounds As Boolean

'Below are resource indexes for sounds that are common to all
'applications.  Numbering for these begins with 200 while constants
'for application-specific sounds are numbered in the 100s.  This
'allows you to add up to 100 sounds that are used with specific
'applications without the need to renumber the indexes below.
'Note:  most of these are not used in this example, nor are there sounds
'       for the unused constants in the resource file.  I STRONGLY
'       encourage everyone to download the Resource Editor from the VB
'       Owner's Area at Microsoft's VB web site.
'       http://www.microsoft.com/vbasic
'       It makes creating a resource file as easy as dragging
'       and dropping a file from Explorer.  :)
Enum SoundIDConstants
    ssThankYou = 200
    ssOptionClick = 201
    ssButtonClick = 202
    ssUhOh = 203
    ssShutDown = 204
    ssStartUp = 205
    ssHelpMe = 206
    ssQuestion = 207
    ssBummer = 208
    ssHelpHomer = 209
    ssOk = 210
    ssCancel = 211
    ssBrowse = 212
    ssPrint = 213
End Enum


Property Let Enabled(OnOrOff As Boolean)

'Assigns whether sounds are on or off
mbPlaySounds = OnOrOff

End Property

Property Get Enabled() As Boolean

'Returns whether sounds are on or off
Enabled = mbPlaySounds

End Property

Public Sub BeginPlaySound(ByVal iResourceId As Integer, Optional ByVal lFlags As Long = ssSndAsync, Optional ByVal bOverride As Boolean = False)

'iResourceId is the index number assigned to the sound in the resource
'file.  To make things easier, create constants for each index
'number.  See the General Declarations section of this class module
'for more information.

'lFlags is an optional argument to pass the ssSnd_ constants to
'sndPlaySound.  If not specified, the default is to play the
'sound asynchronously (control returns immediately), with no
'default sound (if iResourceId is invalid), and from memory (required).

'bOverride is another optional argument to override the Enabled
'property.  If not specifed, the default is False (do not override
'Enabled property).  By specifying True for bOverride, the
'sound will ALWAYS play, even if Enabled is False.  This could be
'useful if you need a sound to play if the user has sounds disabled.

'Buffer to hold the sound in memory.
Dim SoundBuffer As String
Dim lRet As Long

'If both these are False, we will not play a sound
If bOverride = False And mbPlaySounds = False Then
    Exit Sub
End If

On Error GoTo NoPlay

'If we're looping the sound, we have to make SURE that
'the sound is being played asynchronously.
If (lFlags And ssSndLoop) = ssSndLoop Then
    lFlags = lFlags Or ssSndAsync
End If

'Get data from the resource file and
'convert it to a unicode string
SoundBuffer = StrConv(LoadResData(iResourceId, "WAVE"), vbUnicode)

'Call the API function to play the sound
lRet = sndPlaySound(SoundBuffer, lFlags Or ssSndNoDefault Or ssSndMemory)

'Important: This function is necessary for playing sounds asynchronously
lRet = DoEvents()

NoPlay:

End Sub

Public Sub EndPlaySound()

'Invoke this method if you wish to stop playback
'of a sound playing asynchronously (for example,
'a sound that is being looped).

Dim lRet As Long
lRet = sndPlaySound(0&, 0&)

End Sub





